﻿using System;
using System.Threading.Tasks;
using System.Threading;
using WebSocketSharp;
using WebSocketSharp.Server;
using Tobii.Research;
using TrackerBridge;

namespace GazeWebSocketServer
{
    public class Program
    {
        private static GazeServer gazeServer;
        public static void Main(string[] args)
        {
            ConfigurationData.InitializeUnexistingWithDefaults();
            ConfigurationData config = ConfigurationData.ParseToObject();

            TobiiEyeTracker tobiiEyeTracker = new TobiiEyeTracker(config.TrackerSerialNumber);
            tobiiEyeTracker.ConnectionEvent += OnConnectionEstablished;
            tobiiEyeTracker.ConnectionTimeout += OnConnectionTimeout;
            tobiiEyeTracker.GazeDataAvailable += OnGazeDataAvailable;
            tobiiEyeTracker.Connect();

            gazeServer = new GazeServer(config.WebSocketPort, config.WebSocketEndpoint);
            gazeServer.Start();
            Console.ReadLine();
        }

        private static void OnGazeDataAvailable(TobiiEyeTracker sender, GazeData data)
        {
            Console.WriteLine(data);
            gazeServer.Publish(data);
        }

        private static void OnConnectionTimeout(TobiiEyeTracker sender)
        {
            Console.WriteLine("Timeout while searching for trackers.");
        }

        private static void OnConnectionEstablished(TobiiEyeTracker sender)
        {
            Console.WriteLine($"Tracker {sender.ToString()} connected.");
            ConfigurationData config = ConfigurationData.ParseToObject();
            sender.SetTrackingFrequency(config.TrackerFrequency);
        }
    }
}